#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
################################################################################
#                                                                              #
# vidgif                                                                       #
#                                                                              #
################################################################################
#                                                                              #
# LICENCE INFORMATION                                                          #
#                                                                              #
# This program converts a video to an animated GIF.                            #
#                                                                              #
# copyright (C) 2017 Will Breaden Madden, wbm@protonmail.ch                    #
#                                                                              #
# This software is released under the terms of the GNU General Public License  #
# version 3 (GPLv3).                                                           #
#                                                                              #
# This program is free software: you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
# For a copy of the GNU General Public License, see                            #
# <http://www.gnu.org/licenses/>.                                              #
#                                                                              #
################################################################################

usage:
    program [options]

options:
    -h, --help               display help message
    --version                display version and exit
    -v, --verbose            verbose logging
    -s, --silent             silent
    -u, --username=USERNAME  username

    --filename_video=TEXT    filename of input video  [default: video.mp4]
    --filename_GIF=TEXT      filename of output GIF   [default: video.gif]
    --directory=TEXT         directoryname for tmp    [default: tmp]
"""

from __future__ import division

import docopt
import math
import uuid

import propyte
import pymediainfo
import shijian

name     = "vidgif"
version  = "2018-01-17T1511Z"
logo     = None
instance = str(uuid.uuid4())

def main(options):

    global program
    program = propyte.Program(
        options = options,
        name    = name,
        version = version,
        logo    = logo
    )
    global log
    from propyte import log

    filename_video = options["--filename_video"]
    filename_GIF   = options["--filename_GIF"]
    directory_tmp  = options["--directory"]

    media_information = pymediainfo.MediaInfo.parse(filename_video)

    duration = int(math.ceil(media_information.tracks[0].duration / 1000))

    command =\
    """
    export MAGICK_MEMORY_LIMIT=1024
    export MAGICK_MAP_LIMIT=1024
    export MAGICK_AREA_LIMIT=4096
    export MAGICK_FILES_LIMIT=1024
    export MAGICK_THREAD_LIMIT=1
    export MAGICK_TMPDIR=/tmp

    mplayer -ao null -ss 0:00:00 -endpos {duration} {filename_video} -vo jpeg:outdir={directory_tmp}:quality=100

    #mogrify -resize 50% {directory_tmp}/*.jpg

    convert -delay 5 -loop 0 -layers optimize {directory_tmp}/*.jpg {filename_GIF}
    """.format(
        filename_video = filename_video,
        filename_GIF   = filename_GIF,
        directory_tmp  = directory_tmp,
        duration       = duration
    )

    log.info(command)
    shijian.engage_command(command = command)

    program.terminate()

if __name__ == "__main__":
    options = docopt.docopt(__doc__)
    if options["--version"]:
        print(version)
        exit()
    main(options)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
################################################################################
#                                                                              #
# Markdown_to_HTML                                                             #
#                                                                              #
################################################################################
#                                                                              #
# LICENCE INFORMATION                                                          #
#                                                                              #
# This program converts Markdown files to HTML recursively.                    #
#                                                                              #
# copyright (C) 2018 William Breaden Madden                                    #
#                                                                              #
# This software is released under the terms of the GNU General Public License  #
# version 3 (GPLv3).                                                           #
#                                                                              #
# This program is free software: you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
# For a copy of the GNU General Public License, see                            #
# <http://www.gnu.org/licenses/>.                                              #
#                                                                              #
################################################################################

usage:
    program [options]

options:
    -h, --help           display help message
    --version            display version and exit

    --directory=PATH     directory at which to recursively compile files
                         [default: .]
    
    --table_of_contents  include table of contents
    
    --CSS=PATH           path to CSS
                         [default: https://rawgit.com/wdbm/style/master/SS/newswire.css]
"""

import docopt
import os
import subprocess

name    = "Markdown_to_HTML"
version = "2018-01-17T1506Z"

def main(options):

    directory         = options["--directory"]
    table_of_contents = options["--table_of_contents"]
    CSS               = options["--CSS"]
    filepaths = filepaths_recursive(directory = directory)
    filepaths_compile = [filepath for filepath in filepaths if ".md" in filepath]
    for filepath in filepaths_compile:
        filepath_output                                     =\
            os.path.dirname(filepath)                       +\
            "/"                                             +\
            os.path.splitext(os.path.basename(filepath))[0] +\
            ".html"
        print("compile {filepath} to {filepath_output}".format(
            filepath        = filepath,
            filepath_output = filepath_output
        ))
        if table_of_contents:
            command = [
                "pandoc",
                "--table-of-contents",
                "--number-sections",
                "-c",
                CSS,
                filepath,
                "-o",
                filepath_output
            ]
        else:
            command = [
                "pandoc",
                "--number-sections",
                "-c",
                CSS,
                filepath,
                "-o",
                filepath_output
            ]
        process = subprocess.Popen(command, stdout = subprocess.PIPE)
        output, error = process.communicate()

def filepaths_recursive(
    directory = "."
    ):

    """
    Return a list of filepaths found recursively at the specified directory.
    """

    directory = os.path.expanduser(directory)
    filepaths = []
    for root, directories, filenames in os.walk(directory):
        for filename in filenames: 
            filepaths.append(os.path.join(root, filename))
    return filepaths

if __name__ == "__main__":
    options = docopt.docopt(__doc__)
    if options["--version"]:
        print(version)
        exit()
    main(options)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
################################################################################
#                                                                              #
# images_to_video                                                              #
#                                                                              #
################################################################################
#                                                                              #
# LICENCE INFORMATION                                                          #
#                                                                              #
# This program is a way of processing views of the ATLAS Control Room and OP   #
# Vistars pages.                                                               #
#                                                                              #
# copyright (C) 2015 William Breaden Madden                                    #
#                                                                              #
# This software is released under the terms of the GNU General Public License  #
# version 3 (GPLv3).                                                           #
#                                                                              #
# This program is free software: you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
# For a copy of the GNU General Public License, see                            #
# <http://www.gnu.org/licenses/>.                                              #
#                                                                              #
################################################################################

usage:
    program [options]

options:
    -h, --help           display help message
    --version            display version and exit

    --extension=TEXT     input image files extension  [default: png]
    --soundtrack=FILE    soundtrack file              [default: None]
    --output=FILE        output video filename        [default: video.avi]
    --fps=FPS            frames per second            [default: 30]
"""

import docopt
import os
import time
import re

from   moviepy.editor import *
import propyte
import shijian

name    = "images_to_video"
version = "2018-01-17T1525Z"

def main(options):

    codec_video = "png" # "mpeg4"
    codec_audio = "libvorbis"

    extension           =       options["--extension"]
    filename_soundtrack =       options["--soundtrack"]
    filename_output     =       options["--output"]
    FPS                 = float(options["--fps"])

    list_of_image_files = shijian.find_file_sequences(
        extension = extension
    )
    print("list of image files: {list_of_image_files}".format(
        list_of_image_files = list_of_image_files
    ))
    video = ImageSequenceClip(list_of_image_files, fps = FPS)

    propyte.pause(prompt = "Press Enter to write video.")

    if filename_soundtrack == "None":
        video.write_videofile(
            filename_output,
            fps         = FPS,
            codec       = codec_video,
            audio       = False
        )
    else:
        soundtrack = AudioFileClip(filename_soundtrack)
        video = video.set_audio(soundtrack)
        video.write_videofile(
            filename_output,
            fps         = FPS,
            codec       = codec_video,
            audio_codec = codec_audio,
            audio       = True
        )

if __name__ == "__main__":
    options = docopt.docopt(__doc__)
    if options["--version"]:
        print(version)
        exit()
    main(options)
